import static org.junit.Assert.*;

import java.rmi.RemoteException;
import java.util.ArrayList;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import controller.Controller;
import controller.ProjektStatus;
import controller.modelController.ModelController;
import controller.modelController.ResultContent;
import databaseModel.Ansprechpartner;
import databaseModel.Projekt;
import databaseModel.Student;
import databaseModel.Unternehmen;
import databaseModel.AbstractEntity;

/**
 * setUpBeforeClass() und tearDownAfterClass() werden einmal vor/nach allen Tests durchgefhrt,
 * setUp() und tearDown() dagegen vor/nach jedem einzelnen Test.
 * 
 * @author Thomas Kramer, th-rzv@gmx.de
 *
 */
public class BeanValidationTest {
	private static ArrayList<Object> globalObjectCollection = new ArrayList<Object>();
	
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		//new Controller(new String[] {"1099", "192.168.178.45"});		
		new Controller(new String[] {});		
	}
	
	@Before
	public void setUp() throws Exception {
	}
	
	@After
	public void tearDown() throws Exception {		
		/**
		 * bevor die Verbindung zum Anwendungsserver abgebaut wird mssen die Testdatenstze wieder rausgelscht werden.
		 * natrlich in umgekehrter Reihenfolge damit die Foreign Key-Constraints nicht verletzt werden.
		 * 
		 * (natrlich knnte auch das automatische kaskadierende Lschen der Datenbank verwendet werden, solange die 
		 * entsprechenden Constraints gesetzt wurden)
		 * 
		 * So bleibt jedenfalls kein Datensatz ber und es gibt keine Fehlermeldungen weil der Datensatz nicht vorhanden ist.
		 */				
		for (int i = globalObjectCollection.size() - 1; i >= 0; i--) {
			Object test = globalObjectCollection.get(i);
			Class testClass = ((AbstractEntity) test).getClass();
			Integer testId = ((AbstractEntity) test).getId();

			ResultContent rc = ModelController.getDataWithClause(testClass, "id", testId);
			for (Object ob: rc.getListContent())
			{
				ModelController.deleteData(ob);
			}
		}		
		
		globalObjectCollection.clear();
	}	

	@AfterClass
	public static void tearDownAfterClass() throws Exception {		
		/**
		 * abbauen der Verbindung zum Anwendungsserver
		 */
		try {
			Controller.rmiServer.removeObserver(Controller.rmiClient);
		} catch (RemoteException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}			
	}
	
	/**
	 * Testen ob das Einfgen von Standarddatenstzen in die Datenbank funktioniert.
	 * Jede Entity sollte hier aufgenommen werden.
	 */
	@Test
	public void testInsertStandardDataRows() {
		boolean result;
		ArrayList<Object> objectCollection;		
		
		Unternehmen unternehmen = new Unternehmen();
		unternehmen.setName("testUnternehmen");
		
		Student student = new Student();
		student.setName("testStudent");
		student.setEmail("testStudent@gmx.de");
		student.setMat(12345);
		
		Ansprechpartner ansprechpartner = new Ansprechpartner();
		ansprechpartner.setName("testAnsprechpartner");
		ansprechpartner.setEmail("testAnsprechpartner@gmx.de");
		ansprechpartner.setUnternehmenBean(unternehmen);
		
		Projekt projekt = new Projekt();
		projekt.setTitel("testProjekt");
		projekt.setBeschreibung("Beschreibung fr Testprojekt");
		projekt.setStatus(ProjektStatus.offen.toString());
		projekt.setStudent1Bean(student);
		projekt.setAnsprechpartnerBean(ansprechpartner);
		
		objectCollection = new ArrayList<Object>();
		objectCollection.add(unternehmen);
		objectCollection.add(student);
		objectCollection.add(ansprechpartner);
		objectCollection.add(projekt);
		result = ModelController.insertDataCollection(objectCollection);
		
		/**
		 * nur der globalen Lschen-Liste hinzufgen wenn das Speichern tatschlich geklappt hat
		 */
		if (result)
		{
			globalObjectCollection.add(unternehmen);
			globalObjectCollection.add(student);
			globalObjectCollection.add(ansprechpartner);			
			globalObjectCollection.add(projekt);			
		}		
		assertTrue(result);				
	}	
	
	/**
	 * Testen ob das Einfgen von Datenstzen mit zu langen Texten fr die einzelnen Spalten ordnungsgem vom Hibernate Validator geblockt wird. 
	 */
	@Test
	public void testInsertTooLongDataRows() {
		boolean result;
		ArrayList<Object> objectCollection;
		String tooLongString = "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa" +
				               "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa" +
				               "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa";				
		
		Unternehmen unternehmen = new Unternehmen();
		unternehmen.setName(tooLongString);
		
		Student student = new Student();
		student.setName(tooLongString);
		student.setEmail(tooLongString);
		student.setMat(12345);
		
		Ansprechpartner ansprechpartner = new Ansprechpartner();
		ansprechpartner.setName(tooLongString);
		ansprechpartner.setEmail(tooLongString);
		ansprechpartner.setUnternehmenBean(unternehmen);
		
		Projekt projekt = new Projekt();
		projekt.setTitel(tooLongString);
		projekt.setBeschreibung(tooLongString);
		projekt.setStatus(ProjektStatus.offen.toString());		
		projekt.setKommentar(tooLongString);
		projekt.setStudent1Bean(student);
		projekt.setAnsprechpartnerBean(ansprechpartner);
		
		objectCollection = new ArrayList<Object>();
		objectCollection.add(unternehmen);
		objectCollection.add(student);
		objectCollection.add(ansprechpartner);
		objectCollection.add(projekt);
		result = ModelController.insertDataCollection(objectCollection);
		
		/**
		 * nur der globalen Lschen-Liste hinzufgen wenn das Speichern tatschlich geklappt hat
		 */
		if (result)
		{
			globalObjectCollection.add(unternehmen);
			globalObjectCollection.add(student);
			globalObjectCollection.add(ansprechpartner);			
			globalObjectCollection.add(projekt);			
		}				
		assertTrue(!result);				
	}
	
	/**
	 * Testen ob das Einfgen von Datenstzen mit null-Spalten ordnungsgem vom Hibernate Validator geblockt wird. 
	 */
	@Test
	public void testInsertNullDataRows() {
		boolean result;
		ArrayList<Object> objectCollection;				
		
		Unternehmen unternehmen = new Unternehmen();
		unternehmen.setName(null);
		
		Student student = new Student();
		student.setName(null);
		student.setEmail(null);
		//student.setMat(null);
		
		Ansprechpartner ansprechpartner = new Ansprechpartner();
		ansprechpartner.setName(null);
		ansprechpartner.setEmail(null);
		ansprechpartner.setUnternehmenBean(null);
		
		Projekt projekt = new Projekt();
		projekt.setTitel(null);
		projekt.setBeschreibung(null);
		projekt.setKommentar(null);
		projekt.setStudent1Bean(null);
		projekt.setAnsprechpartnerBean(null);
		
		objectCollection = new ArrayList<Object>();
		objectCollection.add(unternehmen);
		objectCollection.add(student);
		objectCollection.add(ansprechpartner);
		objectCollection.add(projekt);
		result = ModelController.insertDataCollection(objectCollection);
		
		/**
		 * nur der globalen Lschen-Liste hinzufgen wenn das Speichern tatschlich geklappt hat
		 */
		if (result)
		{
			globalObjectCollection.add(unternehmen);
			globalObjectCollection.add(student);
			globalObjectCollection.add(ansprechpartner);			
			globalObjectCollection.add(projekt);			
		}				
		assertTrue(!result);				
	}	
	
	/**
	 * Testen ob das Einfgen von Datenstzen mit ungltigen EMail-Adressen ordnungsgem vom Hibernate Validator geblockt wird.
	 * In meinem Domnenmodell gibt es EMail-Adressen nur bei Studenten und Ansprechpartnern.
	 * Ein Unternehmen muss aber eingefgt werden weil es fr einen Ansprechpartner notwendig ist.
	 * Projekte mssen hier aber nicht getestet werden. 
	 */	
	@Test
	public void testInsertNotValidEMailDataRows() {
		boolean result;
		ArrayList<Object> objectCollection;		
		
		Unternehmen unternehmen = new Unternehmen();
		unternehmen.setName("testUnternehmen");
		
		Student student = new Student();
		student.setName("testStudent");
		student.setEmail("testStudent...xxx");
		student.setMat(12345);
		
		Ansprechpartner ansprechpartner = new Ansprechpartner();
		ansprechpartner.setName("testAnsprechpartner");
		ansprechpartner.setEmail("testAnsprechpartner...xxx");
		ansprechpartner.setUnternehmenBean(unternehmen);
		
		objectCollection = new ArrayList<Object>();
		objectCollection.add(unternehmen);
		objectCollection.add(student);
		objectCollection.add(ansprechpartner);			
			
		result = ModelController.insertDataCollection(objectCollection);
		
		/**
		 * nur der globalen Lschen-Liste hinzufgen wenn das Speichern tatschlich geklappt hat
		 */
		if (result)
		{
			globalObjectCollection.add(unternehmen);
			globalObjectCollection.add(student);
			globalObjectCollection.add(ansprechpartner);			
		}
		assertTrue(!result);					
	}	
	
	/**
	 * Testen ob das Einfgen von Datenstzen mit gltigen EMail-Adressen ordnungsgem vom Hibernate Validator erlaubt wird.
	 * In meinem Domnenmodell gibt es EMail-Adressen nur bei Studenten und Ansprechpartnern.
	 * Ein Unternehmen muss aber eingefgt werden weil es fr einen Ansprechpartner notwendig ist.
	 * Projekte mssen hier aber nicht getestet werden. 
	 */	
	@Test
	public void testInsertValidEMailDataRows() {
		boolean result;
		ArrayList<Object> objectCollection;		
		
		Unternehmen unternehmen = new Unternehmen();
		unternehmen.setName("testUnternehmen");
		
		Student student = new Student();
		student.setName("testStudent");
		student.setEmail("testStudent@gmx.de");
		student.setMat(12345);
		
		Ansprechpartner ansprechpartner = new Ansprechpartner();
		ansprechpartner.setName("testAnsprechpartner");
		ansprechpartner.setEmail("testAnsprechpartner@gmx.de");
		ansprechpartner.setUnternehmenBean(unternehmen);
		
		objectCollection = new ArrayList<Object>();
		objectCollection.add(unternehmen);
		objectCollection.add(student);
		objectCollection.add(ansprechpartner);
		
		result = ModelController.insertDataCollection(objectCollection);
		
		/**
		 * nur der globalen Lschen-Liste hinzufgen wenn das Speichern tatschlich geklappt hat
		 */
		if (result)
		{
			globalObjectCollection.add(unternehmen);
			globalObjectCollection.add(student);
			globalObjectCollection.add(ansprechpartner);			
		}		
		assertTrue(result);					
	}			                  	
}
